/*
Modified BSD License

This file is a part of Statistical package and originates from:
http://sf.net/projects/enjomitchsorbit

Copyright (c) 2012, Szymon "Enjo" Ender
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef VECTOR_H
#define VECTOR_H

#include <vector>
#include <string>
namespace EnjoLib
{
class Vector : public std::vector<double>
{
    public:
    std::string Print() const; // TODO
        Vector( const std::vector<double> & init );
        Vector( size_t n );
        Vector();
        virtual ~Vector();

        //! Length of vector
        double Len() const;
        //! Normalised copy of vector
        Vector Norm() const;
        double SumSquares() const;
        double Sum() const;
        double Mean() const;
        Vector AdjustMean() const;

        Vector & operator += (const Vector & par);
        Vector & operator -= (const Vector & par);
        Vector & operator /= (const double f);
        Vector & operator *= (const double f);
        Vector & operator += (const double f);
        Vector & operator -= (const double f);

        Vector operator + (const Vector & par) const;
        Vector operator - (const Vector & par) const;
        Vector operator - () const;
        Vector operator + (const double f) const;
        Vector operator - (const double f) const;
        Vector operator * (const double f) const;
        Vector operator / (const double f) const;
        bool operator > (const Vector & par) const;
        bool operator < (const Vector & par) const;

    protected:
    private:
        void SizesEqual( const Vector & par, const char * functionName ) const;
};

}
std::ostream& operator<< (std::ostream& stream, const EnjoLib::Vector& v);
#endif // VECTOR_H
